#!/bin/sh
#
# date: 2019-05-10, qin.chenglin@panabit.com, QQ:9856306
#
# before use, you must have the following variables of predefined
# 1: APPROOT=${RAMDISK}/app
#

# your app name
APPNAME=dongguanga

# main root
RAMDISK=/usr/ramdisk
# floweye bin
FLOWEYE=${RAMDISK}/bin/floweye

# app root
APPROOT=${RAMDISK}/app
MYROOT=${APPROOT}/${APPNAME}

. /etc/PG.conf

# dir
# config root
CONFIGDIR=${PGETC}/App/${APPNAME}
WEBROOT=${RAMDISK}/app/${APPNAME}/web/user
# DIR_TOKEN=${RAMDISK}/${APPNAME}_token
DIR_TOKEN=${MYROOT}/token



# file
ERROR_DB=${WEBROOT}/error.lst

# var
# ʺŵ½ 2 Сʱʱ
USER_TIME_OUT=7200


# exec
[ ! -d ${DIR_TOKEN} ] && mkdir -p ${DIR_TOKEN}

: ${op_user:=δ֪}
: ${op_io:=null}
: ${op_act:=null}
: ${op_right:=null}

: ${w_logger:=yes}
: ${num_log:=1000}

errdesc()
{
	awk -v msg="${2}" '/^'${1}': /{e=$2;exit;}END{if(e=="")e=msg;print e;}' ${ERROR_DB}
}

presult()
{
	local code=$1
	local data="$2"
	local msg=""
	local txt=""

	: ${code:=0}
	: ${data:=\"\"}

	msg=`errdesc ${code} ""`
	if [ "${3}" != "" ]; then
		if [ "${msg}" = "" ]; then
			msg="${3}"
		else
			msg="${msg} ${3}"
		fi
	fi
	printf '{"msg":"%s","code":%d,"data":%s}' "${msg}" ${code} "${data}"

	[ "${w_logger}" = "yes" ] && logger ${code} "${msg}"
}

parse_json()
{
	local val=`echo $2 | sed 's/.*['\''|"]'$1'['\''|"]:\([^,}]*\).*/\1/'`
	if [ "${val}" = "${2}" ]; then
		echo ""
	else
		echo $val | sed 's/^['\''|"]//g; s/['\''|"]$//g;'
	fi
}

have_right()
{
	local io=$1
	local act=$2
	local right=$3
	local fright=${CONFIGDIR}/right/${right}.lst

	if [ ! -f ${fright} ]; then
		echo 1102
		return 1
	fi

	err=`awk -v act="${act}" '/^'${io}': /{sub(/\r|\n/,"",$NF);for(i=2;i<=NF;i++){if($i==act){print $i;exit;}}}' ${fright}`
	if [ "${err}" != "${act}" ]; then
		echo 1103
		return 1
	fi
	
	return 0
}

auth_right()
{
	local token=$1
	local user=""
	local buildtm=0
	local right=""
	local fright=""
	local cmptm=`date +%s`

	if [ ! -f ${DIR_TOKEN}/${token} ]; then
		presult 1003
		exit 1;
	fi

	user=`cat ${DIR_TOKEN}/${token}`
	buildtm=${user##*:}
	if [ "${buildtm}" = "${user}" ]; then
		buildtm=0
	fi
	op_user=${user%%:*}

	buildtm=$((${cmptm} - ${buildtm}))
	if [ ${buildtm} -gt ${USER_TIME_OUT} ]; then
		rm -f ${DIR_TOKEN}/${token}
		presult 1002
		exit 1;
	fi

	fright=${CONFIGDIR}/user/${op_user}/right
	if [ ! -f ${fright} ]; then
		presult 1100
		exit 1;
	fi
	
	right=`cat ${fright}`
	if [ "${right}" = "" ]; then
		presult 1101
		exit 1;
	fi
	op_right=${right}

	code=`have_right ${op_io} ${op_act} ${op_right}`
	if [ "${?}" != "0" ]; then
		presult ${code}
		exit 1;
	fi
}

logger()
{
	local ret=$1
	local msg="$2"
	local date=`date +%s`
	local log="${DATAPATH}/App/${APPNAME}/op_`date "+%Y-%m-%d"`.log"

	: ${op_user:=δ֪}
	: ${op_io:=null}
	: ${op_act:=null}

	[ ! -d ${DATAPATH}/App/${APPNAME} ] && mkdir -p ${DATAPATH}/App/${APPNAME}
	echo ${op_user} ${op_io} ${op_act} ${ret} ${date} ${REMOTE_ADDR} ${msg} >> ${log}
	
	num=`wc -l ${log} | awk '{printf $1}'`
	if [ ${num} -gt ${num_log} ]; then
		tail -n ${num_log} > ${log}.tmp
		mv ${log}.tmp ${log}
	fi
}
